"""
Components service
"""
import logging
from typing import List

from portal.database.models import Worker
from portal.service import general

log = logging.getLogger(__name__)


def __set_worker_data(worker: Worker, data: dict) -> Worker:
    allowed = ['name', 'url', 'tags', 'portal_secret']
    return general.update_entity(worker, data, allowed=allowed)


def create_worker(**data) -> Worker:
    """Create worker
    Keyword Args:
        name(str): Name of the submission
        url(str): Address where the worker is accessible
    """
    worker = Worker(name=data['name'], url=data['url'])
    __set_worker_data(worker, data)
    log.info(f"[CREATE] Worker({worker.id}): {data}")
    return worker


def update_worker(worker: Worker, data: dict) -> Worker:
    """Update worker
    Args:
        worker(Worker): Worker instance
        data(dict): Data used for the update

    Returns(Worker): Updated worker instance

    """
    log.info(f"[UPDATE] Worker: {worker.id}: {data}")
    return __set_worker_data(worker=worker, data=data)


def delete_worker(worker: Worker):
    """Delete worker
    Args:
        worker(Worker): Worker instance
    """
    log.info(f"[DELETE] Worker: {worker.id} ({worker.name})")
    general.delete_entity(worker)


def find_all_workers() -> List[Worker]:
    """Gets list of tall workers
    Returns(list): List of workers
    """
    return Worker.query.all()
